// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;

namespace Smdn.Net.Pop3 {
  public class PopMessage : IDisposable, IPopMessageNumber {
    public long Number {
      get; private set;
    }

    long IPopMessageNumber.MessageNumber {
      get { return Number; }
    }

    public Stream Body {
      get { CheckDisposed(); return body; }
    }

    public PopMessage(long messageNumber, Stream body)
    {
      if (body == null)
        throw new ArgumentNullException("body");

      this.Number = messageNumber;
      this.body = body;
    }

    ~PopMessage()
    {
      Dispose();
      GC.SuppressFinalize(this);
    }

    public void Dispose()
    {
      if (body != null) {
        body.Close();
        body = null;
      }
    }

    public override string ToString()
    {
      return string.Format("{{Number={0}, Body={1}}}", Number, Body);
    }

    private void CheckDisposed()
    {
      if (body == null)
        throw new ObjectDisposedException(GetType().FullName);
    }

    private Stream body;
  }
}
