using System;
using System.Net;
using System.Text;
using NUnit.Framework;

namespace Smdn.Security.Authentication.Sasl.Client {
  [TestFixture]
  public class AnonymousMechanismTests {
    [Test, ExpectedException(typeof(SaslException))]
    public void TestCredentialNotSet()
    {
      using (var client = new AnonymousMechanism()) {
        byte[] clientResponse;

        client.Exchange(new byte[0], out clientResponse);
      }
    }

    [Test]
    public void TestExchange()
    {
      using (var client = new AnonymousMechanism()) {
        client.Credential = new NetworkCredential("sirhc", "passwordnotused");

        byte[] serverChallenge;
        byte[] clientResponse;

        serverChallenge = new byte[0];

        Assert.AreEqual(SaslExchangeStatus.Succeeded,
                        client.Exchange(serverChallenge, out clientResponse));

        BytesAssert.AreEqual(Convert.FromBase64String("c2lyaGM="),
                             clientResponse);
      }
    }

    [Test]
    public void TestExchangeCredentialUsernameEmpty()
    {
      using (var client = new AnonymousMechanism()) {
        client.Credential = new NetworkCredential();

        byte[] serverChallenge;
        byte[] clientResponse;

        serverChallenge = new byte[0];

        Assert.AreEqual(SaslExchangeStatus.Failed,
                        client.Exchange(serverChallenge, out clientResponse));
        Assert.IsNull(clientResponse);
      }
    }

    [Test]
    public void TestGetInitialResponse()
    {
      using (var client = new AnonymousMechanism()) {
        Assert.IsTrue(client.ClientFirst);

        client.Credential = new NetworkCredential("sirhc", "passwordnotused");

        byte[] initialResponse;

        Assert.AreNotEqual(SaslExchangeStatus.Failed,
                           client.GetInitialResponse(out initialResponse));

        BytesAssert.AreEqual(Convert.FromBase64String("c2lyaGM="),
                             initialResponse);
      }
    }
  }
}
