using System;
using System.Net;
using System.Text;
using NUnit.Framework;

namespace Smdn.Security.Authentication.Sasl.Client {
  [TestFixture]
  public class CRAMMD5MechanismTests {
    [Test, ExpectedException(typeof(SaslException))]
    public void TestCredentialNotSet()
    {
      using (var client = new CRAMMD5Mechanism()) {
        byte[] clientResponse;

        client.Exchange(new byte[0], out clientResponse);
      }
    }

    [Test]
    public void TestExchange()
    {
      using (var client = new CRAMMD5Mechanism()) {
        client.Credential = new NetworkCredential("tim", "tanstaaftanstaaf");

        byte[] serverChallenge;
        byte[] clientResponse;

        serverChallenge = Convert.FromBase64String("PDE4OTYuNjk3MTcwOTUyQHBvc3RvZmZpY2UucmVzdG9uLm1jaS5uZXQ+");

        Assert.AreEqual(SaslExchangeStatus.Succeeded,
                        client.Exchange(serverChallenge, out clientResponse));

        BytesAssert.AreEqual(Convert.FromBase64String("dGltIGI5MTNhNjAyYzdlZGE3YTQ5NWI0ZTZlNzMzNGQzODkw"),
                             clientResponse);
      }
    }

    [Test]
    public void TestExchangeCredentialUsernameEmpty()
    {
      ExchangeEmptyCredentialProperty(new NetworkCredential(null, "tanstaaftanstaaf"));
    }

    [Test]
    public void TestExchangeCredentialPasswordEmpty()
    {
      ExchangeEmptyCredentialProperty(new NetworkCredential("tim", null));
    }

    private void ExchangeEmptyCredentialProperty(NetworkCredential credential)
    {
      using (var client = new CRAMMD5Mechanism()) {
        client.Credential = credential;

        byte[] serverChallenge;
        byte[] clientResponse;

        serverChallenge = Convert.FromBase64String("PDE4OTYuNjk3MTcwOTUyQHBvc3RvZmZpY2UucmVzdG9uLm1jaS5uZXQ+");

        Assert.AreEqual(SaslExchangeStatus.Failed,
                        client.Exchange(serverChallenge, out clientResponse));
        Assert.IsNull(clientResponse);
      }
    }

    [Test, ExpectedException(typeof(NotSupportedException))]
    public void TestGetInitialResponse()
    {
      using (var client = new CRAMMD5Mechanism()) {
        Assert.IsFalse(client.ClientFirst);

        client.Credential = new NetworkCredential("tim", "tanstaaftanstaaf");

        byte[] initialResponse;

        client.GetInitialResponse(out initialResponse);
      }
    }
  }
}
