using System;
using System.Net;
using NUnit.Framework;

namespace Smdn.Security.Authentication.Sasl.Client {
  [TestFixture]
  public class LoginMechanismTests {
    [Test, ExpectedException(typeof(SaslException))]
    public void TestCredentialNotSet()
    {
      using (var client = new LoginMechanism()) {
        byte[] clientResponse;

        client.Exchange(new byte[0], out clientResponse);
      }
    }

    [Test]
    public void TestExchange()
    {
      using (var client = new LoginMechanism()) {
        client.Credential = new NetworkCredential("user", "pass");

        byte[] serverChallenge = new byte[0];
        byte[] clientResponse;

        Assert.AreEqual(SaslExchangeStatus.Continuing,
                        client.Exchange(serverChallenge, out clientResponse));

        BytesAssert.AreEqual(client.Credential.UserName, clientResponse);

        Assert.AreEqual(SaslExchangeStatus.Succeeded,
                        client.Exchange(serverChallenge, out clientResponse));

        BytesAssert.AreEqual(client.Credential.Password, clientResponse);
      }
    }

    [Test]
    public void TestExchangeCredentialUsernameEmpty()
    {
      using (var client = new LoginMechanism()) {
        client.Credential = new NetworkCredential(null, "pass");

        byte[] serverChallenge = new byte[0];
        byte[] clientResponse;

        Assert.AreEqual(SaslExchangeStatus.Failed,
                        client.Exchange(serverChallenge, out clientResponse));
        Assert.IsNull(clientResponse);
      }
    }

    [Test]
    public void TestExchangeCredentialPasswordEmpty()
    {
      using (var client = new LoginMechanism()) {
        client.Credential = new NetworkCredential("user", null);

        byte[] serverChallenge = new byte[0];
        byte[] clientResponse;

        Assert.AreEqual(SaslExchangeStatus.Continuing,
                        client.Exchange(serverChallenge, out clientResponse));

        BytesAssert.AreEqual(client.Credential.UserName, clientResponse);

        Assert.AreEqual(SaslExchangeStatus.Failed,
                        client.Exchange(serverChallenge, out clientResponse));
        Assert.IsNull(clientResponse);
      }
    }

    [Test]
    public void TestGetInitialResponse()
    {
      using (var client = new LoginMechanism()) {
        Assert.IsTrue(client.ClientFirst);

        client.Credential = new NetworkCredential("user", "pass");

        byte[] initialResponse;

        Assert.AreNotEqual(SaslExchangeStatus.Failed,
                           client.GetInitialResponse(out initialResponse));

        BytesAssert.AreEqual(client.Credential.UserName, initialResponse);
      }
    }
  }
}
